@@ -76,7 +76,6 @@ pub struct Client {
     shutdown: Arc<Mutex<bool>>,
 
     /// The options that this `Client` was created using.
-    #[cfg(feature = "jetstream")]
     pub(crate) options: Arc<Options>,
 }
 
@@ -106,17 +105,11 @@ impl Client {
             }),
             server_info: Arc::new(Mutex::new(ServerInfo::default())),
             shutdown: Arc::new(Mutex::new(false)),
-
-            #[cfg(feature = "jetstream")]
             options: Arc::new(options),
         };
 
-        #[cfg(feature = "jetstream")]
         let options = client.options.clone();
 
-        #[cfg(not(feature = "jetstream"))]
-        let options = Arc::new(options);
-
         // Connector for creating the initial connection and reconnecting when
         // it is broken.
         let connector = Connector::new(url, options.clone())?;
